@@ -1,5 +1,5 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 import datetime
 from collections import deque, defaultdict
 from heapq import heappop, heappush
@@ -40,7 +40,9 @@ class Work:
         states={
             'invisible': Eval('type') != 'task',
             }, depends=['type'])
-    duration = fields.Function(fields.Float('Duration'), 'get_function_fields')
+    duration = fields.Function(
+        fields.TimeDelta('Duration', 'company_work_time'),
+        'get_function_fields')
     early_start_time = fields.DateTime("Early Start Time", readonly=True)
     late_start_time = fields.DateTime("Late Start Time", readonly=True)
     early_finish_time = fields.DateTime("Early Finish Time", readonly=True)
@@ -108,12 +110,13 @@ class Work:
                     leafs.add(work.id)
 
                 total_allocation = 0
-                if not work.allocations:
-                    durations[work.id] = (work.effort or 0)
+                if not work.allocations or not work.effort_duration:
+                    durations[work.id] = work.effort_duration
                     continue
                 for allocation in work.allocations:
                     total_allocation += allocation.percentage
-                durations[work.id] = ((work.effort or 0)
+                durations[work.id] = datetime.timedelta(
+                    seconds=work.effort_duration.total_seconds()
                     / (total_allocation / 100.0))
 
             while leafs:
@@ -168,6 +171,12 @@ class Work:
                         })
                 break
 
+    @property
+    def hours(self):
+        if not self.duration:
+            return 0
+        return self.duration.total_seconds() / 60 / 60
+
     @classmethod
     def add_minutes(cls, company, date, minutes):
         minutes = int(round(minutes))
@@ -290,7 +299,7 @@ class Work:
                     if values[work]['early_start_time']:
                         early_finish_time = self.add_hours(work.company,
                                 values[work]['early_start_time'],
-                                work.duration)
+                                work.hours)
                 values[work]['early_finish_time'] = early_finish_time
 
                 # Propagate constraint_start on successors
@@ -310,7 +319,7 @@ class Work:
 
                 # if no sibling continue to walk up the tree
                 if not work2children.get(work.parent):
-                    if not work.parent in values:
+                    if work.parent not in values:
                         values[work.parent] = {}
                     parent = work.parent
 
@@ -328,7 +337,7 @@ class Work:
                 early_start = None
 
             # update values
-            if not work in values:
+            if work not in values:
                 values[work] = {}
             values[work]['early_start_time'] = early_start
 
@@ -370,7 +379,7 @@ class Work:
                     if values[work]['late_finish_time']:
                         late_start_time = self.add_hours(work.company,
                                 values[work]['late_finish_time'],
-                                -work.duration)
+                                -work.hours)
                 values[work]['late_start_time'] = late_start_time
 
                 # Propagate constraint_finish on predecessors
@@ -390,7 +399,7 @@ class Work:
 
                 # if no sibling continue to walk up the tree
                 if not work2children.get(work.parent):
-                    if not work.parent in values:
+                    if work.parent not in values:
                         values[work.parent] = {}
                     parent = work.parent
 
@@ -408,7 +417,7 @@ class Work:
                 late_finish = None
 
             # update values
-            if not work in values:
+            if work not in values:
                 values[work] = {}
             values[work]['late_finish_time'] = late_finish
 
@@ -499,7 +508,7 @@ class Work:
                     elif item[1] >= delay + sibling.duration:
                         overloaded.append(item)
                     else:
-                        #Succes!
+                        # Succes!
                         break
 
                 heappush(timeline,
